@@ -79,12 +79,15 @@ survey_mean.tbl_svy <- function(
   proportion = FALSE, prop_method = c("logit", "likelihood", "asin", "beta", "mean"),
   deff = FALSE, df = NULL, .svy = current_svy(), ...
 ) {
-  vartype <- if (missing(vartype)) "se" else match.arg(vartype, several.ok = TRUE)
+  if (!is.null(vartype)) {
+    vartype <- if (missing(vartype)) "se" else match.arg(vartype, several.ok = TRUE)
+  }
   prop_method <- match.arg(prop_method)
   if (is.null(df)) df <- survey::degf(.svy)
+  if (missing(x)) stop("Variable should be provided as an argument to survey_mean() or grouped survey object should be used.")
   stop_for_factor(x)
   if (!proportion) {
-    if (class(x) == "logical") x <- as.integer(x)
+    if (is.logical(x)) x <- as.integer(x)
     .svy <- set_survey_vars(.svy, x)
     stat <- survey::svymean(~`__SRVYR_TEMP_VAR__`, .svy, na.rm = na.rm, deff = deff)
     out <- get_var_est(stat, vartype, level = level, df = df, deff = deff)
@@ -107,8 +110,9 @@ survey_mean.grouped_svy <- function(
   proportion = FALSE, prop_method = c("logit", "likelihood", "asin", "beta", "mean"),
   deff = FALSE, df = NULL, .svy = current_svy(), ...
 ) {
-  if (missing(vartype)) vartype <- "se"
-  vartype <- match.arg(vartype, several.ok = TRUE)
+  if (!is.null(vartype)) {
+    vartype <- if (missing(vartype)) "se" else match.arg(vartype, several.ok = TRUE)
+  }
   if (is.null(df)) df <- survey::degf(.svy)
 
   if (missing(prop_method)) prop_method <- "logit"
@@ -120,7 +124,7 @@ survey_mean.grouped_svy <- function(
     survey_stat_factor(.svy, survey::svymean, na.rm, vartype, level, deff, df)
   } else {
     stop_for_factor(x)
-    if (class(x) == "logical") x <- as.integer(x)
+    if (is.logical(x)) x <- as.integer(x)
     .svy <- set_survey_vars(.svy, x)
     grps_formula <- survey::make.formula(group_vars(.svy))
 
@@ -147,7 +151,6 @@ survey_mean.grouped_svy <- function(
   }
 }
 
-
 #' Calculate the total and its variation using survey methods
 #'
 #' Calculate totals from complex survey data. A wrapper
@@ -204,7 +207,7 @@ survey_mean.grouped_svy <- function(
 #'
 #' @export
 survey_total <- function(
-  x = NULL, na.rm = FALSE, vartype = c("se", "ci", "var", "cv"), level = 0.95,
+  x, na.rm = FALSE, vartype = c("se", "ci", "var", "cv"), level = 0.95,
   deff = FALSE, df = NULL, .svy = current_svy(), ...
 ) {
   UseMethod("survey_total", .svy)
@@ -215,17 +218,15 @@ survey_total.tbl_svy <- function(
   x, na.rm = FALSE, vartype = c("se", "ci", "var", "cv"), level = 0.95,
   deff = FALSE, df = NULL, .svy = current_svy(), ...
 ) {
-  if (missing(vartype)) vartype <- "se"
-  vartype <- match.arg(vartype, several.ok = TRUE)
+  if (!is.null(vartype)) {
+    vartype <- if (missing(vartype)) "se" else match.arg(vartype, several.ok = TRUE)
+  }
 
   if (is.null(df)) df <- survey::degf(.svy)
 
-  if (class(x) == "factor") {
-    stop(paste0(
-      "Factor not allowed in survey functions, they should be used as a grouping variable"
-    ))
-  }
-  if (class(x) == "logical") x <- as.integer(x)
+  if (missing(x)) stop("Variable should be provided as an argument to survey_total() or grouped survey object should be used.")
+  stop_for_factor(x)
+  if (is.logical(x)) x <- as.integer(x)
 
   .svy <- set_survey_vars(.svy, x)
   stat <- survey::svytotal(~`__SRVYR_TEMP_VAR__`, .svy, na.rm = na.rm, deff = deff)
@@ -239,15 +240,16 @@ survey_total.grouped_svy <- function(
   x, na.rm = FALSE, vartype = c("se", "ci", "var", "cv"), level = 0.95,
   deff = FALSE, df = NULL, .svy = current_svy(), ...
 ) {
-  if (missing(vartype)) vartype <- "se"
-  vartype <- match.arg(vartype, several.ok = TRUE)
+  if (!is.null(vartype)) {
+    vartype <- if (missing(vartype)) "se" else match.arg(vartype, several.ok = TRUE)
+  }
   if (is.null(df)) df <- survey::degf(.svy)
 
   if (missing(x)) {
     survey_stat_factor(.svy, survey::svytotal, na.rm, vartype, level, deff, df)
   } else {
     stop_for_factor(x)
-    if (class(x) == "logical") x <- as.integer(x)
+    if (is.logical(x)) x <- as.integer(x)
     .svy <- set_survey_vars(.svy, x)
     grps_formula <- survey::make.formula(group_vars(.svy))
 
@@ -262,7 +264,6 @@ survey_total.grouped_svy <- function(
   }
 }
 
-
 #' Calculate the ratio and its variation using survey methods
 #'
 #' Calculate ratios from complex survey data. A wrapper
@@ -326,9 +327,12 @@ survey_ratio.tbl_svy <- function(
   numerator, denominator, na.rm = FALSE, vartype = c("se", "ci", "var", "cv"),
   level = 0.95, deff = FALSE, df = NULL, .svy = current_svy(), ...
 ) {
-  if (missing(vartype)) vartype <- "se"
-  vartype <- match.arg(vartype, several.ok = TRUE)
+  if (!is.null(vartype)) {
+    vartype <- if (missing(vartype)) "se" else match.arg(vartype, several.ok = TRUE)
+  }
   if (is.null(df)) df <- survey::degf(.svy)
+  stop_for_factor(numerator)
+  stop_for_factor(denominator)
 
   .svy <- set_survey_vars(.svy, numerator, "__SRVYR_TEMP_NUM__")
   .svy <- set_survey_vars(.svy, denominator, "__SRVYR_TEMP_DEN__", add = TRUE)
@@ -347,10 +351,12 @@ survey_ratio.grouped_svy <- function(
   numerator, denominator, na.rm = FALSE, vartype = c("se", "ci", "var", "cv"),
   level = 0.95, deff = FALSE, df = NULL, .svy = current_svy(), ...
 ) {
-  if (missing(vartype)) vartype <- "se"
-  vartype <- match.arg(vartype, several.ok = TRUE)
-
+  if (!is.null(vartype)) {
+    vartype <- if (missing(vartype)) "se" else match.arg(vartype, several.ok = TRUE)
+  }
   if (is.null(df)) df <- survey::degf(.svy)
+  stop_for_factor(numerator)
+  stop_for_factor(denominator)
 
   grp_names <- group_vars(.svy)
 
@@ -377,7 +383,7 @@ survey_ratio.grouped_svy <- function(
 #' @param na.rm A logical value to indicate whether missing values should be dropped
 #' @param quantiles A vector of quantiles to calculate
 #' @param vartype NULL to report no variability (default), otherwise one or more of:
-#'                standard error ("se) confidence interval ("ci") (variance and coefficient
+#'                standard error ("se") confidence interval ("ci") (variance and coefficient
 #'                of variation not available).
 #' @param level A single number indicating the confidence level (only one level allowed)
 #' @param q_method See "method" in \code{\link[stats]{approxfun}}
@@ -417,13 +423,14 @@ survey_quantile <- function(
 
 #' @export
 survey_quantile.tbl_svy <- function(
-  x, quantiles, na.rm = FALSE, vartype = NULL,
+  x, quantiles, na.rm = FALSE, vartype = c("se", "ci"),
   level = 0.95, q_method = "linear", f = 1,
   interval_type = c("Wald", "score", "betaWald", "probability", "quantile"),
   ties = c("discrete", "rounded"), df = Inf, .svy = current_svy(), ...
 ) {
-  if (!is.null(vartype)) vartype <- match.arg(vartype, c("se", "ci"), several.ok = TRUE)
-
+  if (!is.null(vartype)) {
+    vartype <- if (missing(vartype)) "se" else match.arg(vartype, several.ok = TRUE)
+  }
   if (missing(interval_type) & !inherits(.svy, "svyrep.design")) interval_type <- "Wald"
   if (missing(interval_type) & inherits(.svy, "svyrep.design")) interval_type <- "probability"
   interval_type <- match.arg(interval_type, several.ok = TRUE)
@@ -441,6 +448,8 @@ survey_quantile.tbl_svy <- function(
   # we could go higher, but I worry about 32bit vs 64bit systems)
   alpha = round(1 - level, 7)
 
+  if (missing(x)) stop("Variable should be provided as an argument to survey_quantile().")
+  stop_for_factor(x)
   .svy <- set_survey_vars(.svy, x)
 
   stat <- survey::svyquantile(
@@ -460,8 +469,9 @@ survey_quantile.grouped_svy <- function(
   interval_type = c("Wald", "score", "betaWald", "probability", "quantile"),
   ties = c("discrete", "rounded"), df = Inf, .svy = current_svy(), ...
 ) {
-  if (!is.null(vartype)) vartype <- match.arg(vartype, c("se", "ci"), several.ok = TRUE)
-
+  if (!is.null(vartype)) {
+    vartype <- if (missing(vartype)) "se" else match.arg(vartype, several.ok = TRUE)
+  }
   if (missing(interval_type) & !inherits(.svy, "svyrep.design")) interval_type <- "Wald"
   if (missing(interval_type) & inherits(.svy, "svyrep.design")) interval_type <- "probability"
   interval_type <- match.arg(interval_type, several.ok = TRUE)
@@ -480,6 +490,8 @@ survey_quantile.grouped_svy <- function(
   alpha = round(1 - level, 7)
 
   grp_names <- group_vars(.svy)
+  if (missing(x)) stop("survey_quantile() can't be used with regards to the grouping variable.")
+  stop_for_factor(x)
   .svy <- set_survey_vars(.svy, x)
 
   stat <- survey::svyby(
@@ -495,16 +507,17 @@ survey_quantile.grouped_svy <- function(
   out
 }
 
-
 #' @export
 #' @rdname survey_quantile
 survey_median <- function(
-  x, na.rm = FALSE, vartype = NULL,
+  x, na.rm = FALSE, vartype = c("se", "ci"),
   level = 0.95, q_method = "linear", f = 1,
   interval_type = c("Wald", "score", "betaWald", "probability", "quantile"),
   ties = c("discrete", "rounded"), df = Inf, .svy = current_svy(), ...
 ) {
-  if (!is.null(vartype)) vartype <- match.arg(vartype, c("se", "ci"), several.ok = TRUE)
+  if (!is.null(vartype)) {
+    vartype <- if (missing(vartype)) "se" else match.arg(vartype, several.ok = TRUE)
+  }
   if (missing(interval_type) & !inherits(.svy, "svyrep.design")) interval_type <- "Wald"
   if (missing(interval_type) & inherits(.svy, "svyrep.design")) interval_type <- "probability"
   interval_type <- match.arg(interval_type, several.ok = TRUE)
@@ -516,10 +529,142 @@ survey_median <- function(
     level <- level[1]
   }
 
-  survey_quantile(
+  out = survey_quantile(
     x, quantiles = 0.5, na.rm = na.rm, vartype = vartype, level = level, q_method = q_method,
     f = f, interval_type = interval_type, ties = ties, df = df, .svy = .svy
   )
+  names(out) = sub("^_q50", "", names(out))
+  out
+}
+
+#' Calculate the population variance and its variation using survey methods
+#'
+#' Calculate population variance from complex survey data. A wrapper
+#' around \code{\link[survey]{svyvar}}. \code{survey_var} should always be
+#' called from \code{\link{summarise}}.
+#'
+#' @param x A variable or expression, or empty
+#' @param na.rm A logical value to indicate whether missing values should be dropped
+#' @param vartype Report variability as one or more of: standard error ("se", default)
+#'                or variance ("var") (confidence intervals and coefficient
+#'                of variation not available).
+#' @param level (For vartype = "ci" only) A single number or vector of numbers indicating
+#'              the confidence level.
+#' @param df (For vartype = "ci" only) A numeric value indicating the degrees of freedom
+#'           for t-distribution. The default (Inf) is equivalent to using normal
+#'           distribution and in case of population variance statistics there is little
+#'           reason to use any other values (see \emph{Details}).
+#' @param .svy A \code{tbl_svy} object. When called from inside a summarize function
+#'             the default automatically sets the survey to the current survey.
+#' @param ... Ignored
+#' @details
+#' Be aware that confidence intervals for population variance statistic are
+#' computed by package \emph{survey} using \emph{t} or normal (with df=Inf)
+#' distribution (i.e. symmetric distributions). \strong{This could be a very poor
+#' approximation} if even one of these conditions is met:
+#' \itemize{
+#'   \item{there are few sampling design degrees of freedom,}
+#'   \item{analyzed variable isn't normally distributed,}
+#'   \item{there is huge variation in sampling probabilities of the survey design.}
+#' }
+#' Because of this be very careful using confidence intervals for population variance
+#' statistics especially while performing analysis within subsets of data or using
+#' grouped survey objects.
+#'
+#' Sampling distribution of the variance statistic in general is asymmetric
+#' (chi-squared in case of simple random sampling of normally distributed variable)
+#' and if analyzed variable isn't normally distributed or there is huge variation in
+#' sampling probabilities of the survey design (or both) it could converge to
+#' normality only very slowly (with growing number of survey design degrees of
+#' freedom).
+#' @examples
+#' library(survey)
+#' data(api)
+#'
+#' dstrata <- apistrat %>%
+#'   as_survey_design(strata = stype, weights = pw)
+#'
+#' dstrata %>%
+#'   summarise(api99_var = survey_var(api99),
+#'             api99_sd = survey_sd(api99))
+#'
+#' dstrata %>%
+#'   group_by(awards) %>%
+#'   summarise(api00_var = survey_var(api00),
+#'             api00_sd = survey_sd(api00))
+#'
+#' # standard deviation and variance of the population variance estimator
+#' # are available with vartype argument
+#' # (but not for the population standard deviation estimator)
+#' dstrata %>%
+#'   summarise(api99_variance = survey_var(api99, vartype = c("se", "var")))
+#' @export
+
+survey_var <- function(
+  x, na.rm = FALSE, vartype = c("se", "ci", "var"), level = 0.95, df = Inf,
+  .svy = current_svy(), ...
+) {
+  UseMethod("survey_var", .svy)
+}
+
+#' @export
+survey_var.tbl_svy <- function(
+  x, na.rm = FALSE, vartype = c("se", "ci", "var"), level = 0.95, df = Inf,
+  .svy = current_svy(), ...
+) {
+  if (!is.null(vartype)) {
+    vartype <- if (missing(vartype)) "se" else match.arg(vartype, several.ok = TRUE)
+  }
+  if (missing(x)) stop("Variable should be provided as an argument to survey_var().")
+  stop_for_factor(x)
+  if (is.logical(x)) x <- as.integer(x)
+
+  .svy <- set_survey_vars(.svy, x)
+  stat <- survey::svyvar(~`__SRVYR_TEMP_VAR__`, .svy, na.rm = na.rm)
+
+  out <- get_var_est(stat, vartype, level = level, df = df, deff = FALSE)
+  out
+}
+
+#' @export
+survey_var.grouped_svy <- function(
+  x, na.rm = FALSE, vartype = c("se", "ci", "var"), level = 0.95, df = Inf,
+  .svy = current_svy(), ...
+) {
+  if (!is.null(vartype)) {
+    vartype <- if (missing(vartype)) "se" else match.arg(vartype, several.ok = TRUE)
+  }
+  if (missing(x)) stop("survey_var() can't be used with regards to the grouping variable.")
+  stop_for_factor(x)
+  if (is.logical(x)) x <- as.integer(x)
+  .svy <- set_survey_vars(.svy, x)
+  grps_formula <- survey::make.formula(group_vars(.svy))
+
+  if (any(dplyr::count(.svy$variables)$n < 2)) stop("Population variance can't be computed because some groups contain less than 2 observations.")
+
+  stat <- survey::svyby(
+    ~`__SRVYR_TEMP_VAR__`, grps_formula, .svy, survey::svyvar,
+    na.rm = na.rm
+  )
+  out <- get_var_est(
+    stat, vartype, grps = group_vars(.svy), level = level, df = df, deff = FALSE
+  )
+  out
+}
+
+#' @export
+#' @rdname survey_var
+survey_sd <- function(
+  x, na.rm = FALSE, .svy = current_svy(), ...
+) {
+  out <- survey_var(
+    x, na.rm = na.rm, vartype = NULL, .svy = .svy
+  )
+  out <- mutate(
+    out,
+    `__SRVYR_COEF__` = sqrt(.data$`__SRVYR_COEF__`)
+  )
+  out
 }
 
 #' Calculate the an unweighted summary statistic from a survey
@@ -556,7 +701,6 @@ unweighted <- function(x, .svy = current_svy(), ...) {
   out
 }
 
-
 survey_stat_factor <- function(.svy, func, na.rm, vartype, level, deff, df) {
   grps_names <- group_vars(.svy)
   .svy <- set_survey_vars(.svy, NULL)
